@@ -16,42 +16,45 @@
 CLI interface for nova status commands.
 """
 
-import collections
 import functools
 import sys
 import traceback
 
 from keystoneauth1 import exceptions as ks_exc
 from oslo_config import cfg
-from oslo_serialization import jsonutils
+from oslo_upgradecheck import common_checks
 from oslo_upgradecheck import upgradecheck
-from oslo_utils import fileutils
 import pkg_resources
-import six
+import sqlalchemy as sa
 from sqlalchemy import func as sqlfunc
-from sqlalchemy import MetaData, Table, and_, select
 
 from nova.cmd import common as cmd_common
 import nova.conf
 from nova import config
 from nova import context as nova_context
-from nova.db.sqlalchemy import api as db_session
+from nova.db.api import api as api_db_api
+from nova.db.main import api as main_db_api
 from nova import exception
 from nova.i18n import _
 from nova.objects import cell_mapping as cell_mapping_obj
+# NOTE(lyarwood): The following are imported as machine_type_utils expects them
+# to be registered under nova.objects when called via _check_machine_type_set
+from nova.objects import image_meta as image_meta_obj  # noqa: F401
+from nova.objects import instance as instance_obj  # noqa: F401
 from nova import policy
 from nova import utils
 from nova import version
+from nova.virt.libvirt import machine_type_utils
 from nova.volume import cinder
 
 CONF = nova.conf.CONF
 
-# NOTE(efried): 1.35 is required by nova-scheduler to support the root_required
-# queryparam to make GET /allocation_candidates require that a trait be present
-# on the root provider, irrespective of how the request groups are specified.
+# NOTE(gibi): 1.36 is required by nova-scheduler to support the same_subtree
+# queryparam to make GET /allocation_candidates require that a list of request
+# groups are satisfied from the same provider subtree.
 # NOTE: If you bump this version, remember to update the history
 # section in the nova-status man page (doc/source/cli/nova-status).
-MIN_PLACEMENT_MICROVERSION = "1.35"
+MIN_PLACEMENT_MICROVERSION = "1.36"
 
 # NOTE(mriedem): 3.44 is needed to work with volume attachment records which
 # are required for supporting multi-attach capable volumes.
@@ -84,10 +87,15 @@ class UpgradeCommands(upgradecheck.UpgradeCommands):
         # table, or by only counting compute nodes with a service version of at
         # least 15 which was the highest service version when Newton was
         # released.
-        meta = MetaData(bind=db_session.get_engine(context=context))
-        compute_nodes = Table('compute_nodes', meta, autoload=True)
-        return select([sqlfunc.count()]).select_from(compute_nodes).where(
-                compute_nodes.c.deleted == 0).scalar()
+        meta = sa.MetaData()
+        engine = main_db_api.get_engine(context=context)
+        compute_nodes = sa.Table('compute_nodes', meta, autoload_with=engine)
+        with engine.connect() as conn:
+            return conn.execute(
+                sa.select(sqlfunc.count()).select_from(compute_nodes).where(
+                    compute_nodes.c.deleted == 0
+                )
+            ).scalars().first()
 
     def _check_cellsv2(self):
         """Checks to see if cells v2 has been setup.
@@ -100,8 +108,8 @@ class UpgradeCommands(upgradecheck.UpgradeCommands):
         this on an initial install. This also has to be careful about checking
         for compute nodes if there are no host mappings on a fresh install.
         """
-        meta = MetaData()
-        meta.bind = db_session.get_api_engine()
+        meta = sa.MetaData()
+        engine = api_db_api.get_engine()
 
         cell_mappings = self._get_cell_mappings()
         count = len(cell_mappings)
@@ -120,8 +128,13 @@ class UpgradeCommands(upgradecheck.UpgradeCommands):
                     'retry.')
             return upgradecheck.Result(upgradecheck.Code.FAILURE, msg)
 
-        host_mappings = Table('host_mappings', meta, autoload=True)
-        count = select([sqlfunc.count()]).select_from(host_mappings).scalar()
+        host_mappings = sa.Table('host_mappings', meta, autoload_with=engine)
+
+        with engine.connect() as conn:
+            count = conn.execute(
+                sa.select(sqlfunc.count()).select_from(host_mappings)
+            ).scalars().first()
+
         if count == 0:
             # This may be a fresh install in which case there may not be any
             # compute_nodes in the cell database if the nova-compute service
@@ -190,15 +203,7 @@ class UpgradeCommands(upgradecheck.UpgradeCommands):
         return upgradecheck.Result(upgradecheck.Code.SUCCESS)
 
     @staticmethod
-    def _get_non_cell0_mappings():
-        """Queries the API database for non-cell0 cell mappings.
-
-        :returns: list of nova.objects.CellMapping objects
-        """
-        return UpgradeCommands._get_cell_mappings(include_cell0=False)
-
-    @staticmethod
-    def _get_cell_mappings(include_cell0=True):
+    def _get_cell_mappings():
         """Queries the API database for cell mappings.
 
         .. note:: This method is unique in that it queries the database using
@@ -210,114 +215,12 @@ class UpgradeCommands(upgradecheck.UpgradeCommands):
                   formatted 'database_connection' value back on those objects
                   (they are read-only).
 
-        :param include_cell0: True if cell0 should be returned, False if cell0
-            should be excluded from the results.
         :returns: list of nova.objects.CellMapping objects
         """
         ctxt = nova_context.get_admin_context()
         cell_mappings = cell_mapping_obj.CellMappingList.get_all(ctxt)
-        if not include_cell0:
-            # Since CellMappingList does not implement remove() we have to
-            # create a new list and exclude cell0.
-            mappings = [mapping for mapping in cell_mappings
-                        if not mapping.is_cell0()]
-            cell_mappings = cell_mapping_obj.CellMappingList(objects=mappings)
         return cell_mappings
 
-    @staticmethod
-    def _is_ironic_instance_migrated(extras, inst):
-        extra = (extras.select().where(extras.c.instance_uuid == inst['uuid']
-                                       ).execute().first())
-        # Pull the flavor and deserialize it. Note that the flavor info for an
-        # instance is a dict keyed by "cur", "old", "new" and we want the
-        # current flavor.
-        flavor = jsonutils.loads(extra['flavor'])['cur']['nova_object.data']
-        # Do we have a custom resource flavor extra spec?
-        specs = flavor['extra_specs'] if 'extra_specs' in flavor else {}
-        for spec_key in specs:
-            if spec_key.startswith('resources:CUSTOM_'):
-                # We found a match so this instance is good.
-                return True
-        return False
-
-    def _check_ironic_flavor_migration(self):
-        """In Pike, ironic instances and flavors need to be migrated to use
-        custom resource classes. In ironic, the node.resource_class should be
-        set to some custom resource class value which should match a
-        "resources:<custom resource class name>" flavor extra spec on baremetal
-        flavors. Existing ironic instances will have their embedded
-        instance.flavor.extra_specs migrated to use the matching ironic
-        node.resource_class value in the nova-compute service, or they can
-        be forcefully migrated using "nova-manage db ironic_flavor_migration".
-
-        In this check, we look for all ironic compute nodes in all non-cell0
-        cells, and from those ironic compute nodes, we look for an instance
-        that has a "resources:CUSTOM_*" key in it's embedded flavor extra
-        specs.
-        """
-        cell_mappings = self._get_non_cell0_mappings()
-        ctxt = nova_context.get_admin_context()
-        # dict of cell identifier (name or uuid) to number of unmigrated
-        # instances
-        unmigrated_instance_count_by_cell = collections.defaultdict(int)
-        for cell_mapping in cell_mappings:
-            with nova_context.target_cell(ctxt, cell_mapping) as cctxt:
-                # Get the (non-deleted) ironic compute nodes in this cell.
-                meta = MetaData(bind=db_session.get_engine(context=cctxt))
-                compute_nodes = Table('compute_nodes', meta, autoload=True)
-                ironic_nodes = (
-                    compute_nodes.select().where(and_(
-                        compute_nodes.c.hypervisor_type == 'ironic',
-                        compute_nodes.c.deleted == 0
-                    )).execute().fetchall())
-
-                if ironic_nodes:
-                    # We have ironic nodes in this cell, let's iterate over
-                    # them looking for instances.
-                    instances = Table('instances', meta, autoload=True)
-                    extras = Table('instance_extra', meta, autoload=True)
-                    for node in ironic_nodes:
-                        nodename = node['hypervisor_hostname']
-                        # Get any (non-deleted) instances for this node.
-                        ironic_instances = (
-                            instances.select().where(and_(
-                                instances.c.node == nodename,
-                                instances.c.deleted == 0
-                            )).execute().fetchall())
-                        # Get the instance_extras for each instance so we can
-                        # find the flavors.
-                        for inst in ironic_instances:
-                            if not self._is_ironic_instance_migrated(
-                                    extras, inst):
-                                # We didn't find the extra spec key for this
-                                # instance so increment the number of
-                                # unmigrated instances in this cell.
-                                unmigrated_instance_count_by_cell[
-                                    cell_mapping.uuid] += 1
-
-        if not cell_mappings:
-            # There are no non-cell0 mappings so we can't determine this, just
-            # return a warning. The cellsv2 check would have already failed
-            # on this.
-            msg = (_('Unable to determine ironic flavor migration without '
-                     'cell mappings.'))
-            return upgradecheck.Result(upgradecheck.Code.WARNING, msg)
-
-        if unmigrated_instance_count_by_cell:
-            # There are unmigrated ironic instances, so we need to fail.
-            msg = (_('There are (cell=x) number of unmigrated instances in '
-                     'each cell: %s. Run \'nova-manage db '
-                     'ironic_flavor_migration\' on each cell.') %
-                   ' '.join('(%s=%s)' % (
-                       cell_id, unmigrated_instance_count_by_cell[cell_id])
-                            for cell_id in
-                            sorted(unmigrated_instance_count_by_cell.keys())))
-            return upgradecheck.Result(upgradecheck.Code.FAILURE, msg)
-
-        # Either there were no ironic compute nodes or all instances for
-        # those nodes are already migrated, so there is nothing to do.
-        return upgradecheck.Result(upgradecheck.Code.SUCCESS)
-
     def _check_cinder(self):
         """Checks to see that the cinder API is available at a given minimum
         microversion.
@@ -343,7 +246,7 @@ class UpgradeCommands(upgradecheck.UpgradeCommands):
             return upgradecheck.Result(
                 upgradecheck.Code.WARNING,
                 _('Unable to determine Cinder API version due to error: %s') %
-                six.text_type(ex))
+                str(ex))
         return upgradecheck.Result(upgradecheck.Code.SUCCESS)
 
     def _check_policy(self):
@@ -376,7 +279,7 @@ class UpgradeCommands(upgradecheck.UpgradeCommands):
                 "2. Use a pre-existing sample config file from the Train "
                 "release. 3. Use an empty or non-existent file to take all "
                 "the defaults.")
-        rule = "system_admin_api"
+        rule = "context_is_admin"
         rule_new_default = "role:admin and system_scope:all"
         status = upgradecheck.Result(upgradecheck.Code.SUCCESS)
         # NOTE(gmann): Initialise the policy if it not initialized.
@@ -404,29 +307,12 @@ class UpgradeCommands(upgradecheck.UpgradeCommands):
             status = upgradecheck.Result(
                 upgradecheck.Code.WARNING,
                 _('Unable to perform policy checks due to error: %s') %
-                six.text_type(ex))
+                str(ex))
         # reset the policy state so that it can be initialized from fresh if
         # operator changes policy file after running this upgrade checks.
         policy.reset()
         return status
 
-    def _check_policy_json(self):
-        "Checks to see if policy file is JSON-formatted policy file."
-        msg = _("Your policy file  is JSON-formatted which is "
-                "deprecated since Victoria release (Nova 22.0.0). "
-                "You need to switch to YAML-formatted file. You can use the "
-                "``oslopolicy-convert-json-to-yaml`` tool to convert existing "
-                "JSON-formatted files to YAML-formatted files in a "
-                "backwards-compatible manner: "
-                "https://docs.openstack.org/oslo.policy/"
-                "latest/cli/oslopolicy-convert-json-to-yaml.html.")
-        status = upgradecheck.Result(upgradecheck.Code.SUCCESS)
-        # Check if policy file exist and is JSON-formatted.
-        policy_path = CONF.find_file(CONF.oslo_policy.policy_file)
-        if policy_path and fileutils.is_json(policy_path):
-            status = upgradecheck.Result(upgradecheck.Code.FAILURE, msg)
-        return status
-
     def _check_old_computes(self):
         # warn if there are computes in the system older than the previous
         # major release
@@ -437,6 +323,28 @@ class UpgradeCommands(upgradecheck.UpgradeCommands):
 
         return upgradecheck.Result(upgradecheck.Code.SUCCESS)
 
+    def _check_machine_type_set(self):
+        ctxt = nova_context.get_admin_context()
+        if machine_type_utils.get_instances_without_type(ctxt):
+            msg = (_("""
+Instances found without hw_machine_type set. This warning can be ignored if
+your environment does not contain libvirt based compute hosts.
+Use the `nova-manage machine_type list_unset` command to list these instances.
+For more details see the following:
+https://docs.openstack.org/latest/nova/admin/hw_machine_type.html"""))
+            return upgradecheck.Result(upgradecheck.Code.WARNING, msg)
+
+        return upgradecheck.Result(upgradecheck.Code.SUCCESS)
+
+    def _check_service_user_token(self):
+        if not CONF.service_user.send_service_user_token:
+            msg = (_("""
+Service user token configuration is required for all Nova services.
+For more details see the following:
+https://docs.openstack.org/latest/nova/admin/configuration/service-user-token.html"""))  # noqa
+            return upgradecheck.Result(upgradecheck.Code.FAILURE, msg)
+        return upgradecheck.Result(upgradecheck.Code.SUCCESS)
+
     # The format of the check functions is to return an upgradecheck.Result
     # object with the appropriate upgradecheck.Code and details set. If the
     # check hits warnings or failures then those should be stored in the
@@ -449,16 +357,21 @@ class UpgradeCommands(upgradecheck.UpgradeCommands):
         (_('Cells v2'), _check_cellsv2),
         # Added in Ocata
         (_('Placement API'), _check_placement),
-        # Added in Rocky (but also useful going back to Pike)
-        (_('Ironic Flavor Migration'), _check_ironic_flavor_migration),
         # Added in Train
         (_('Cinder API'), _check_cinder),
         # Added in Ussuri
         (_('Policy Scope-based Defaults'), _check_policy),
         # Added in Victoria
-        (_('Policy File JSON to YAML Migration'), _check_policy_json),
-        # Backported from Wallaby
-        (_('Older than N-1 computes'), _check_old_computes)
+        (
+            _('Policy File JSON to YAML Migration'),
+            (common_checks.check_policy_json, {'conf': CONF})
+        ),
+        # Added in Wallaby
+        (_('Older than N-1 computes'), _check_old_computes),
+        # Added in Wallaby
+        (_('hw_machine_type unset'), _check_machine_type_set),
+        # Added in Bobcat
+        (_('Service User Token Configuration'), _check_service_user_token),
     )
 
 
