@@ -23,7 +23,6 @@ SHOULD include dedicated exception logging.
 """
 
 from oslo_log import log as logging
-import six
 
 import webob.exc
 from webob import util as woutil
@@ -82,7 +81,7 @@ class NovaException(Exception):
             if not message:
                 message = self.msg_fmt % kwargs
             else:
-                message = six.text_type(message)
+                message = str(message)
         except Exception:
             # NOTE(melwitt): This is done in a separate method so it can be
             # monkey-patched during testing to make it a hard failure.
@@ -136,6 +135,10 @@ class GlanceConnectionFailed(NovaException):
         "%(reason)s")
 
 
+class KeystoneConnectionFailed(NovaException):
+    msg_fmt = _("Connection to keystone host failed: %(reason)s")
+
+
 class CinderConnectionFailed(NovaException):
     msg_fmt = _("Connection to cinder host failed: %(reason)s")
 
@@ -156,9 +159,29 @@ class Forbidden(NovaException):
     code = 403
 
 
-class ForbiddenWithAccelerators(NovaException):
-    msg_fmt = _("Forbidden with instances that have accelerators.")
-    code = 403
+class NotSupported(NovaException):
+    # This exception use return code as 400 and can be used
+    # directly or as base exception for operations whihc are not
+    # supported in Nova. Any feature that is not yet implemented
+    # but plan to implement in future (example: Cyborg
+    # integration operations), should use this exception as base
+    # and override the msg_fmt with feature details.
+    # Example: MultiattachNotSupportedByVirtDriver exception.
+    msg_fmt = _("Bad Request - Feature is not supported in Nova")
+    code = 400
+
+
+class ForbiddenWithAccelerators(NotSupported):
+    msg_fmt = _("Feature not supported with instances that have accelerators.")
+
+
+class ForbiddenPortsWithAccelerator(NotSupported):
+    msg_fmt = _("Feature not supported with Ports that have accelerators.")
+
+
+class ForbiddenWithRemoteManagedPorts(NotSupported):
+    msg_fmt = _("This feature is not supported when remote-managed ports"
+                " are in use.")
 
 
 class AdminRequired(Forbidden):
@@ -274,14 +297,18 @@ class VolumeDetachFailed(Invalid):
                 "Reason: %(reason)s")
 
 
-class MultiattachNotSupportedByVirtDriver(NovaException):
+class VolumeExtendFailed(Invalid):
+    msg_fmt = _("Volume %(volume_id)s could not be extended. "
+                "Reason: %(reason)s")
+
+
+class MultiattachNotSupportedByVirtDriver(NotSupported):
     # This exception indicates the compute hosting the instance does not
     # support multiattach volumes. This should generally be considered a
-    # 409 HTTPConflict error in the API since we expect all virt drivers to
+    # 400 HTTPBadRequest error in the API since we expect all virt drivers to
     # eventually support multiattach volumes.
     msg_fmt = _("Volume %(volume_id)s has 'multiattach' set, "
                 "which is not supported for this instance.")
-    code = 409
 
 
 class MultiattachNotSupportedOldMicroversion(Invalid):
@@ -494,6 +521,10 @@ class MultiplePortsNotApplicable(Invalid):
     msg_fmt = _("Failed to launch instances: %(reason)s")
 
 
+class AmbiguousHostnameForMultipleInstances(Invalid):
+    msg_fmt = _("Unable to allocate a single hostname to multiple instances")
+
+
 class InvalidFixedIpAndMaxCountRequest(Invalid):
     msg_fmt = _("Failed to launch instances: %(reason)s")
 
@@ -527,16 +558,21 @@ class UnableToMigrateToSelf(Invalid):
                 "to current host (%(host)s).")
 
 
-class OperationNotSupportedForSEV(NovaException):
+class OperationNotSupportedForSEV(NotSupported):
     msg_fmt = _("Operation '%(operation)s' not supported for SEV-enabled "
                 "instance (%(instance_uuid)s).")
-    code = 409
 
 
-class OperationNotSupportedForVTPM(NovaException):
+class OperationNotSupportedForVTPM(NotSupported):
     msg_fmt = _("Operation '%(operation)s' not supported for vTPM-enabled "
                 "instance (%(instance_uuid)s).")
-    code = 409
+
+
+class OperationNotSupportedForVDPAInterface(NotSupported):
+    msg_fmt = _(
+        "Operation '%(operation)s' not supported for instance with "
+        "vDPA ports ((instance_uuid)s)."
+    )
 
 
 class InvalidHypervisorType(Invalid):
@@ -641,15 +677,6 @@ class NotFound(NovaException):
     code = 404
 
 
-class AgentBuildNotFound(NotFound):
-    msg_fmt = _("No agent-build associated with id %(id)s.")
-
-
-class AgentBuildExists(NovaException):
-    msg_fmt = _("Agent-build with hypervisor %(hypervisor)s os %(os)s "
-                "architecture %(architecture)s exists.")
-
-
 class VolumeAttachmentNotFound(NotFound):
     msg_fmt = _("Volume attachment %(attachment_id)s could not be found.")
 
@@ -704,10 +731,19 @@ class VolumeDriverNotFound(NotFound):
     msg_fmt = _("Could not find a handler for %(driver_type)s volume.")
 
 
+class VolumeDriverNotSupported(VolumeDriverNotFound):
+    msg_fmt = _("The %(volume_driver)s volume driver is not supported on this "
+                "platform.")
+
+
 class InvalidImageRef(Invalid):
     msg_fmt = _("Invalid image href %(image_href)s.")
 
 
+class InvalidImagePropertyName(Invalid):
+    msg_fmt = _("Invalid image property name %(image_property_name)s.")
+
+
 class AutoDiskConfigDisabledByImage(Invalid):
     msg_fmt = _("Requested image %(image)s "
                 "has automatic disk resize disabled.")
@@ -814,13 +850,13 @@ class PortBindingFailed(Invalid):
 
 
 class PortBindingDeletionFailed(NovaException):
-    msg_fmt = _("Failed to delete binding for port %(port_id)s and host "
-                "%(host)s.")
+    msg_fmt = _("Failed to delete binding for port(s) %(port_id)s on host "
+                "%(host)s; please check neutron logs for more information")
 
 
 class PortBindingActivationFailed(NovaException):
-    msg_fmt = _("Failed to activate binding for port %(port_id)s and host "
-                "%(host)s.")
+    msg_fmt = _("Failed to activate binding for port %(port_id)s on host "
+                "%(host)s; please check neutron logs for more information")
 
 
 class PortUpdateFailed(Invalid):
@@ -970,10 +1006,6 @@ class QuotaClassExists(NovaException):
     msg_fmt = _("Quota class %(class_name)s exists for resource %(resource)s")
 
 
-class OverQuota(NovaException):
-    msg_fmt = _("Quota exceeded for resources: %(overs)s")
-
-
 class SecurityGroupNotFound(NotFound):
     msg_fmt = _("Security group %(security_group_id)s not found.")
 
@@ -1201,33 +1233,35 @@ class RequestFilterFailed(NovaException):
     msg_fmt = _("Scheduling failed: %(reason)s")
 
 
+class InvalidRoutedNetworkConfiguration(NovaException):
+    msg_fmt = _("Neutron routed networks configuration is invalid: "
+                "%(reason)s.")
+
+
 class MaxRetriesExceeded(NoValidHost):
     msg_fmt = _("Exceeded maximum number of retries. %(reason)s")
 
 
-class QuotaError(NovaException):
-    msg_fmt = _("Quota exceeded: code=%(code)s")
-    # NOTE(cyeoh): 413 should only be used for the ec2 API
-    # The error status code for out of quota for the nova api should be
-    # 403 Forbidden.
+class OverQuota(NovaException):
+    msg_fmt = _("Quota exceeded for resources: %(overs)s")
     code = 413
     safe = True
 
 
-class TooManyInstances(QuotaError):
+class TooManyInstances(OverQuota):
     msg_fmt = _("Quota exceeded for %(overs)s: Requested %(req)s,"
                 " but already used %(used)s of %(allowed)s %(overs)s")
 
 
-class FloatingIpLimitExceeded(QuotaError):
+class FloatingIpLimitExceeded(OverQuota):
     msg_fmt = _("Maximum number of floating IPs exceeded")
 
 
-class MetadataLimitExceeded(QuotaError):
+class MetadataLimitExceeded(OverQuota):
     msg_fmt = _("Maximum number of metadata items exceeds %(allowed)d")
 
 
-class OnsetFileLimitExceeded(QuotaError):
+class OnsetFileLimitExceeded(OverQuota):
     msg_fmt = _("Personality file limit exceeded")
 
 
@@ -1239,22 +1273,24 @@ class OnsetFileContentLimitExceeded(OnsetFileLimitExceeded):
     msg_fmt = _("Personality file content exceeds maximum %(allowed)s")
 
 
-class KeypairLimitExceeded(QuotaError):
-    msg_fmt = _("Maximum number of key pairs exceeded")
+class KeypairLimitExceeded(OverQuota):
+    msg_fmt = _("Quota exceeded, too many key pairs.")
 
 
-class SecurityGroupLimitExceeded(QuotaError):
+class SecurityGroupLimitExceeded(OverQuota):
     msg_fmt = _("Maximum number of security groups or rules exceeded")
 
 
-class PortLimitExceeded(QuotaError):
+class PortLimitExceeded(OverQuota):
     msg_fmt = _("Maximum number of ports exceeded")
 
 
-# TODO(stephenfin): Remove this XenAPI relic
-class AggregateError(NovaException):
-    msg_fmt = _("Aggregate %(aggregate_id)s: action '%(action)s' "
-                "caused an error: %(reason)s.")
+class ServerGroupLimitExceeded(OverQuota):
+    msg_fmt = _("Quota exceeded, too many server groups.")
+
+
+class GroupMemberLimitExceeded(OverQuota):
+    msg_fmt = _("Quota exceeded, too many servers in group")
 
 
 class AggregateNotFound(NotFound):
@@ -1355,6 +1391,11 @@ class InterfaceAttachPciClaimFailed(Invalid):
                 "interface attach")
 
 
+class InterfaceAttachResourceAllocationFailed(Invalid):
+    msg_fmt = _("Failed to allocate additional resources to %(instance_uuid)s "
+                "during interface attach")
+
+
 class InterfaceDetachFailed(Invalid):
     msg_fmt = _("Failed to detach network adapter device from "
                 "%(instance_uuid)s")
@@ -1436,8 +1477,13 @@ class RescheduledException(NovaException):
                 "%(reason)s")
 
 
-class ShadowTableExists(NovaException):
-    msg_fmt = _("Shadow table with name %(name)s already exists.")
+class RescheduledByPolicyException(RescheduledException):
+    msg_fmt = _("Build of instance %(instance_uuid)s was re-scheduled: "
+                "%(reason)s")
+
+
+class GroupAffinityViolation(NovaException):
+    msg_fmt = _("%(policy)s instance group policy was violated")
 
 
 class InstanceFaultRollback(NovaException):
@@ -1542,6 +1588,16 @@ class PciRequestFromVIFNotFound(NotFound):
                 "PCI address: %(pci_slot)s on compute node: %(node_id)s")
 
 
+class PciDeviceRemoteManagedNotPresent(NovaException):
+    msg_fmt = _('Invalid PCI Whitelist: A device specified as "remote_managed"'
+                ' is not actually present on the host')
+
+
+class PciDeviceInvalidPFRemoteManaged(NovaException):
+    msg_fmt = _('Invalid PCI Whitelist: PFs must not have the "remote_managed"'
+                'tag, device address: %(address)s')
+
+
 # Cannot be templated, msg needs to be constructed when raised.
 class InternalError(NovaException):
     """Generic hypervisor errors.
@@ -1551,11 +1607,6 @@ class InternalError(NovaException):
     msg_fmt = "%(err)s"
 
 
-class PciDevicePrepareFailed(NovaException):
-    msg_fmt = _("Failed to prepare PCI device %(id)s for instance "
-                "%(instance_uuid)s: %(reason)s")
-
-
 class PciDeviceDetachFailed(NovaException):
     msg_fmt = _("Failed to detach PCI device %(dev)s: %(reason)s")
 
@@ -1758,6 +1809,15 @@ class InvalidMachineType(Invalid):
                 "%(image_name)s (%(image_id)s): %(reason)s")
 
 
+class InvalidMachineTypeUpdate(Invalid):
+    msg_fmt = _("Cannot update machine type %(existing_machine_type)s to "
+                "%(machine_type)s.")
+
+
+class UnsupportedMachineType(Invalid):
+    msg_fmt = _("Machine type %(machine_type)s is not supported.")
+
+
 class InvalidVirtualMachineMode(Invalid):
     msg_fmt = _("Virtual machine mode '%(vmmode)s' is not recognised")
 
@@ -1834,11 +1894,6 @@ class ImageCPUThreadPolicyForbidden(Forbidden):
                 "override CPU thread pinning policy set against the flavor")
 
 
-class ImagePMUConflict(Forbidden):
-    msg_fmt = _("Image property 'hw_pmu' is not permitted to "
-                "override the PMU policy set in the flavor")
-
-
 class UnsupportedPolicyException(Invalid):
     msg_fmt = _("ServerGroup policy is not supported: %(reason)s")
 
@@ -1889,6 +1944,14 @@ class UEFINotSupported(Invalid):
     msg_fmt = _("UEFI is not supported")
 
 
+class SecureBootNotSupported(Invalid):
+    msg_fmt = _("Secure Boot is not supported by host")
+
+
+class FirmwareSMMNotSupported(Invalid):
+    msg_fmt = _("This firmware doesn't require (support) SMM")
+
+
 class TriggerCrashDumpNotSupported(Invalid):
     msg_fmt = _("Triggering crash dump is not supported")
 
@@ -1925,6 +1988,13 @@ class AttachInterfaceWithQoSPolicyNotSupported(AttachInterfaceNotSupported):
                 "instance %(instance_uuid)s.")
 
 
+class AttachWithExtendedQoSPolicyNotSupported(AttachInterfaceNotSupported):
+    msg_fmt = _(
+        "The interface attach server operation with port having extended "
+        "resource request, like a port with both QoS minimum bandwidth and "
+        "packet rate policies, is not yet supported.")
+
+
 class NetworksWithQoSPolicyNotSupported(Invalid):
     msg_fmt = _("Using networks with QoS policy is not supported for "
                 "instance %(instance_uuid)s. (Network ID is %(network_id)s)")
@@ -1936,6 +2006,12 @@ class CreateWithPortResourceRequestOldVersion(Invalid):
                 "until microversion 2.72.")
 
 
+class ExtendedResourceRequestOldCompute(Invalid):
+    msg_fmt = _("The port-resource-request-groups neutron API extension is "
+                "not supported by old nova compute service. Upgrade your "
+                "compute services to Xena (24.0.0) or later.")
+
+
 class InvalidReservedMemoryPagesOption(Invalid):
     msg_fmt = _("The format of the option 'reserved_huge_pages' is invalid. "
                 "(found '%(conf)s') Please refer to the nova "
@@ -2024,11 +2100,6 @@ class UsagesRetrievalFailed(NovaException):
                 "user '%(user_id)s'.")
 
 
-class UnsupportedPointerModelRequested(Invalid):
-    msg_fmt = _("Pointer model '%(model)s' requested is not supported by "
-                "host.")
-
-
 class NotSupportedWithOption(Invalid):
     msg_fmt = _("%(operation)s is not supported in conjunction with the "
                 "current %(option)s setting.  Please refer to the nova "
@@ -2230,32 +2301,28 @@ class MissingDomainCapabilityFeatureException(NovaException):
                 "including <%(feature)s> feature.")
 
 
+class HealAllocationException(NovaException):
+    msg_fmt = _("Healing instance allocation failed.")
+
+
+class HealvGPUAllocationNotSupported(HealAllocationException):
+    msg_fmt = _(
+        "Healing allocation for instance %(instance_uuid)s with vGPU resource "
+        "request is not supported."
+    )
+
+
+class HealDeviceProfileAllocationNotSupported(HealAllocationException):
+    msg_fmt = _(
+        "Healing allocation for instance %(instance_uuid)s with Cyborg device "
+        "profile request is not supported."
+    )
+
+
 class HealPortAllocationException(NovaException):
     msg_fmt = _("Healing port allocation failed.")
 
 
-class MoreThanOneResourceProviderToHealFrom(HealPortAllocationException):
-    msg_fmt = _("More than one matching resource provider %(rp_uuids)s is "
-                "available for healing the port allocation for port "
-                "%(port_id)s for instance %(instance_uuid)s. This script "
-                "does not have enough information to select the proper "
-                "resource provider from which to heal.")
-
-
-class NoResourceProviderToHealFrom(HealPortAllocationException):
-    msg_fmt = _("No matching resource provider is "
-                "available for healing the port allocation for port "
-                "%(port_id)s for instance %(instance_uuid)s. There are no "
-                "resource providers with matching traits %(traits)s in the "
-                "provider tree of the resource provider %(node_uuid)s ."
-                "This probably means that the neutron QoS configuration is "
-                "wrong. Consult with "
-                "https://docs.openstack.org/neutron/latest/admin/"
-                "config-qos-min-bw.html for information on how to configure "
-                "neutron. If the configuration is fixed the script can be run "
-                "again.")
-
-
 class UnableToQueryPorts(HealPortAllocationException):
     msg_fmt = _("Unable to query ports for instance %(instance_uuid)s: "
                 "%(error)s")
@@ -2327,13 +2394,14 @@ class AcceleratorRequestOpFailed(NovaException):
 class AcceleratorRequestBindingFailed(NovaException):
     msg_fmt = _("Failed to bind accelerator requests: %(msg)s")
 
-    def __init__(self, arqs, **kwargs):
-        super().__init__(message=self.msg_fmt, **kwargs)
+    def __init__(self, message=None, arqs=None, **kwargs):
+        super(AcceleratorRequestBindingFailed, self).__init__(
+            message=message, **kwargs)
         self.arqs = arqs or []
 
 
-class InvalidLibvirtGPUConfig(NovaException):
-    msg_fmt = _('Invalid configuration for GPU devices: %(reason)s')
+class InvalidLibvirtMdevConfig(NovaException):
+    msg_fmt = _('Invalid configuration for mdev-capable devices: %(reason)s')
 
 
 class RequiredMixedInstancePolicy(Invalid):
