@@ -20,17 +20,15 @@ from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_utils import timeutils
 from oslo_utils import versionutils
-from sqlalchemy import or_
-from sqlalchemy.sql import false
+import sqlalchemy as sa
+from sqlalchemy import sql
 from sqlalchemy.sql import func
-from sqlalchemy.sql import null
 
 from nova import availability_zones as avail_zone
 from nova.compute import task_states
 from nova.compute import vm_states
-from nova.db import api as db
-from nova.db.sqlalchemy import api as db_api
-from nova.db.sqlalchemy import models
+from nova.db.main import api as db
+from nova.db.main import models
 from nova import exception
 from nova.i18n import _
 from nova.network import model as network_model
@@ -148,6 +146,8 @@ class Instance(base.NovaPersistentObject, base.NovaObject,
         'host': fields.StringField(nullable=True),
         'node': fields.StringField(nullable=True),
 
+        # TODO(stephenfin): Remove this in version 3.0 of the object as it has
+        # been replaced by 'flavor'
         'instance_type_id': fields.IntegerField(nullable=True),
 
         'user_data': fields.StringField(nullable=True),
@@ -186,7 +186,8 @@ class Instance(base.NovaPersistentObject, base.NovaObject,
         'shutdown_terminate': fields.BooleanField(default=False),
         'disable_terminate': fields.BooleanField(default=False),
 
-        # TODO(stephenfin): Remove this in version 3.0 of the object
+        # TODO(stephenfin): Remove this in version 3.0 of the object as it's
+        # related to cells v1
         'cell_name': fields.StringField(nullable=True),
 
         'metadata': fields.DictOfStringsField(),
@@ -1088,6 +1089,11 @@ class Instance(base.NovaPersistentObject, base.NovaObject,
     def obj_load_attr(self, attrname):
         # NOTE(danms): We can't lazy-load anything without a context and a uuid
         if not self._context:
+            if 'uuid' in self:
+                LOG.debug(
+                    "Lazy-load of '%s' attempted by orphaned instance",
+                    attrname, instance=self
+                )
             raise exception.OrphanedObjectError(method='obj_load_attr',
                                                 objtype=self.obj_name())
         if 'uuid' not in self:
@@ -1225,10 +1231,6 @@ class Instance(base.NovaPersistentObject, base.NovaObject,
             pci_req for pci_req in self.pci_requests.requests
             if pci_req.request_id != pci_device.request_id]
 
-    def add_pci_device_and_request(self, pci_device, pci_request):
-        self.pci_requests.requests.append(pci_request)
-        self.pci_devices.objects.append(pci_device)
-
 
 def _make_instance_list(context, inst_list, db_inst_list, expected_attrs):
     get_fault = expected_attrs and 'fault' in expected_attrs
@@ -1257,7 +1259,7 @@ def _make_instance_list(context, inst_list, db_inst_list, expected_attrs):
     return inst_list
 
 
-@db_api.pick_context_manager_writer
+@db.pick_context_manager_writer
 def populate_missing_availability_zones(context, count):
     # instances without host have no reasonable AZ to set
     not_empty_host = models.Instance.host != None  # noqa E711
@@ -1347,7 +1349,7 @@ class InstanceList(base.ObjectListBase, base.NovaObject):
                                    expected_attrs)
 
     @staticmethod
-    @db_api.pick_context_manager_reader
+    @db.pick_context_manager_reader
     def _get_uuids_by_host_and_node(context, host, node):
         return context.session.query(
             models.Instance.uuid).filter_by(
@@ -1498,7 +1500,7 @@ class InstanceList(base.ObjectListBase, base.NovaObject):
         return db.instance_get_all_uuids_by_hosts(context, hosts)
 
     @staticmethod
-    @db_api.pick_context_manager_reader
+    @db.pick_context_manager_reader
     def _get_count_by_vm_state_in_db(context, project_id, user_id, vm_state):
         return context.session.query(models.Instance.id).\
             filter_by(deleted=0).\
@@ -1513,14 +1515,14 @@ class InstanceList(base.ObjectListBase, base.NovaObject):
                                                 vm_state)
 
     @staticmethod
-    @db_api.pick_context_manager_reader
+    @db.pick_context_manager_reader
     def _get_counts_in_db(context, project_id, user_id=None):
-        # NOTE(melwitt): Copied from nova/db/sqlalchemy/api.py:
+        # NOTE(melwitt): Copied from nova/db/main/api.py:
         # It would be better to have vm_state not be nullable
         # but until then we test it explicitly as a workaround.
-        not_soft_deleted = or_(
+        not_soft_deleted = sa.or_(
             models.Instance.vm_state != vm_states.SOFT_DELETED,
-            models.Instance.vm_state == null()
+            models.Instance.vm_state == sql.null()
             )
         project_query = context.session.query(
             func.count(models.Instance.id),
@@ -1533,8 +1535,10 @@ class InstanceList(base.ObjectListBase, base.NovaObject):
         # non-hidden version of the instance in another cell database and the
         # API will only show one of them, so we don't count the hidden copy.
         project_query = project_query.filter(
-            or_(models.Instance.hidden == false(),
-                models.Instance.hidden == null()))
+            sa.or_(
+                models.Instance.hidden == sql.false(),
+                models.Instance.hidden == sql.null(),
+            ))
 
         project_result = project_query.first()
         fields = ('instances', 'cores', 'ram')
@@ -1568,7 +1572,7 @@ class InstanceList(base.ObjectListBase, base.NovaObject):
         return cls._get_counts_in_db(context, project_id, user_id=user_id)
 
     @staticmethod
-    @db_api.pick_context_manager_reader
+    @db.pick_context_manager_reader
     def _get_count_by_hosts(context, hosts):
         return context.session.query(models.Instance).\
             filter_by(deleted=0).\
