@@ -61,6 +61,7 @@ from nova import exception
 from nova import objects
 from nova.objects import base as objects_base
 from nova import quota
+from nova.scheduler.client import report
 from nova.tests import fixtures as nova_fixtures
 from nova.tests.unit import matchers
 from nova import utils
@@ -290,6 +291,9 @@ class TestCase(base.BaseTestCase):
         # instead of only once initialized for test worker
         wsgi_app.init_global_data.reset()
 
+        # Reset the placement client singleton
+        report.PLACEMENTCLIENT = None
+
     def _setup_cells(self):
         """Setup a normal cellsv2 environment.
 
@@ -388,7 +392,7 @@ class TestCase(base.BaseTestCase):
             engine = db_api.get_engine()
         dialect = engine.url.get_dialect()
         if dialect == sqlite.dialect:
-            engine.connect().execute("PRAGMA foreign_keys = ON")
+            engine.connect().exec_driver_sql("PRAGMA foreign_keys = ON")
 
     def start_service(self, name, host=None, cell_name=None, **kwargs):
         # Disallow starting multiple scheduler services
@@ -779,6 +783,7 @@ class MatchType(object):
             "world",
             MatchType(objects.KeyPair))
     """
+
     def __init__(self, wanttype):
         self.wanttype = wanttype
 
@@ -794,6 +799,7 @@ class MatchType(object):
 
 class MatchObjPrims(object):
     """Matches objects with equal primitives."""
+
     def __init__(self, want_obj):
         self.want_obj = want_obj
 
@@ -823,6 +829,7 @@ class ContainKeyValue(object):
             "world",
             ContainKeyValue('hello', world))
     """
+
     def __init__(self, wantkey, wantvalue):
         self.wantkey = wantkey
         self.wantvalue = wantvalue
