@@ -15,18 +15,16 @@
 
 import mock
 from oslo_config import cfg
-import six
 
 from nova.compute import instance_actions
 from nova import context
-from nova.db import api as db
-from nova.db.sqlalchemy import api as db_api
+from nova.db.main import api as db
+from nova import objects
 from nova import test
 from nova.tests import fixtures as nova_fixtures
 from nova.tests.functional.api import client
 from nova.tests.functional import fixtures as func_fixtures
 from nova.tests.functional import integrated_helpers
-from nova.tests.unit import policy_fixture
 from nova import utils
 from nova.virt import fake
 
@@ -62,17 +60,17 @@ class ServerGroupTestBase(test.TestCase,
         self.flags(weight_classes=self._get_weight_classes(),
                    group='filter_scheduler')
 
-        self.useFixture(policy_fixture.RealPolicyFixture())
+        self.useFixture(nova_fixtures.RealPolicyFixture())
         self.useFixture(nova_fixtures.GlanceFixture(self))
         self.useFixture(nova_fixtures.NeutronFixture(self))
 
         self.useFixture(func_fixtures.PlacementFixture())
-        api_fixture = self.useFixture(nova_fixtures.OSAPIFixture(
+        self.api_fixture = self.useFixture(nova_fixtures.OSAPIFixture(
             api_version='v2.1'))
 
-        self.api = api_fixture.api
+        self.api = self.api_fixture.api
         self.api.microversion = self.microversion
-        self.admin_api = api_fixture.admin_api
+        self.admin_api = self.api_fixture.admin_api
         self.admin_api.microversion = self.microversion
 
         self.start_service('conductor')
@@ -177,13 +175,8 @@ class ServerGroupTestV21(ServerGroupTestBase):
 
         # Create an API using project 'openstack1'.
         # This is a non-admin API.
-        #
-        # NOTE(sdague): this is actually very much *not* how this
-        # fixture should be used. This actually spawns a whole
-        # additional API server. Should be addressed in the future.
-        api_openstack1 = self.useFixture(nova_fixtures.OSAPIFixture(
-            api_version=self.api_major_version,
-            project_id=PROJECT_ID_ALT)).api
+        api_openstack1 = self.api_fixture.alternative_api
+        api_openstack1.project_id = PROJECT_ID_ALT
         api_openstack1.microversion = self.microversion
 
         # Create a server group in project 'openstack'
@@ -263,9 +256,9 @@ class ServerGroupTestV21(ServerGroupTestBase):
         ctxt = context.get_admin_context()
         servers = db.instance_get_all(ctxt)
         self.assertEqual(1, len(servers))
-        ctxt_mgr = db_api.get_context_manager(ctxt)
+        ctxt_mgr = db.get_context_manager(ctxt)
         with ctxt_mgr.reader.using(ctxt):
-            bdms = db_api._block_device_mapping_get_query(ctxt).all()
+            bdms = db._block_device_mapping_get_query(ctxt).all()
         self.assertEqual(1, len(bdms))
         self.assertEqual(servers[0]['uuid'], bdms[0]['instance_uuid'])
 
@@ -446,10 +439,9 @@ class ServerGroupTestV21(ServerGroupTestBase):
         # Start additional host to test evacuation
         self.start_service('compute', host='host3')
 
-        post = {'evacuate': {'onSharedStorage': False}}
-        self.admin_api.post_server_action(servers[1]['id'], post)
-        self._wait_for_migration_status(servers[1], ['done'])
-        evacuated_server = self._wait_for_state_change(servers[1], 'ACTIVE')
+        evacuated_server = self._evacuate_server(
+            servers[1], {'onSharedStorage': 'False'},
+            expected_migration_status='done')
 
         # check that the server is evacuated to another host
         self.assertNotEqual(evacuated_server['OS-EXT-SRV-ATTR:host'],
@@ -467,11 +459,9 @@ class ServerGroupTestV21(ServerGroupTestBase):
         # Set forced_down on the host to ensure nova considers the host down.
         self._set_forced_down(host, True)
 
-        post = {'evacuate': {'onSharedStorage': False}}
-        self.admin_api.post_server_action(servers[1]['id'], post)
-        self._wait_for_migration_status(servers[1], ['error'])
-        server_after_failed_evac = self._wait_for_state_change(
-            servers[1], 'ERROR')
+        server_after_failed_evac = self._evacuate_server(
+            servers[1], {'onSharedStorage': 'False'}, expected_state='ERROR',
+            expected_migration_status='error')
 
         # assert that after a failed evac the server active on the same host
         # as before
@@ -487,11 +477,9 @@ class ServerGroupTestV21(ServerGroupTestBase):
         # Set forced_down on the host to ensure nova considers the host down.
         self._set_forced_down(host, True)
 
-        post = {'evacuate': {'onSharedStorage': False}}
-        self.admin_api.post_server_action(servers[1]['id'], post)
-        self._wait_for_migration_status(servers[1], ['error'])
-        server_after_failed_evac = self._wait_for_state_change(
-            servers[1], 'ERROR')
+        server_after_failed_evac = self._evacuate_server(
+            servers[1], {'onSharedStorage': 'False'}, expected_state='ERROR',
+            expected_migration_status='error')
 
         # assert that after a failed evac the server active on the same host
         # as before
@@ -507,6 +495,85 @@ class ServerGroupTestV21(ServerGroupTestBase):
         self.assertIn('Invalid input', ex.response.text)
         self.assertIn('soft-affinity', ex.response.text)
 
+    @mock.patch('nova.scheduler.filters.affinity_filter.'
+        'ServerGroupAffinityFilter.host_passes', return_value=True)
+    def test_failed_count_with_affinity_violation(self, mock_host_passes):
+        """Check failed count not incremented after violation of the late
+        affinity check. https://bugs.launchpad.net/nova/+bug/1996732
+        """
+
+        created_group = self.api.post_server_groups(self.affinity)
+        flavor = self.api.get_flavors()[2]
+
+        # Ensure the first instance is on compute1
+        with utils.temporary_mutation(self.admin_api, microversion='2.53'):
+            compute2_service_id = self.admin_api.get_services(
+            host=self.compute2.host, binary='nova-compute')[0]['id']
+            self.admin_api.put_service(compute2_service_id,
+                                        {'status': 'disabled'})
+
+        self._boot_a_server_to_group(created_group, flavor=flavor)
+
+        # Ensure the second instance is on compute2
+        with utils.temporary_mutation(self.admin_api, microversion='2.53'):
+            self.admin_api.put_service(compute2_service_id,
+                                        {'status': 'enabled'})
+            compute1_service_id = self.admin_api.get_services(
+            host=self.compute.host, binary='nova-compute')[0]['id']
+            self.admin_api.put_service(compute1_service_id,
+                                        {'status': 'disabled'})
+
+        # Expects GroupAffinityViolation exception
+        failed_server = self._boot_a_server_to_group(created_group,
+                                                     flavor=flavor,
+                                                     expected_status='ERROR')
+
+        self.assertEqual('Exceeded maximum number of retries. Exhausted all '
+                         'hosts available for retrying build failures for '
+                         'instance %s.' % failed_server['id'],
+                         failed_server['fault']['message'])
+
+        ctxt = context.get_admin_context()
+        computes = objects.ComputeNodeList.get_all(ctxt)
+
+        for node in computes:
+            self.assertEqual(node.stats.get('failed_builds'), '0')
+
+    @mock.patch('nova.scheduler.filters.affinity_filter.'
+        'ServerGroupAntiAffinityFilter.host_passes', return_value=True)
+    def test_failed_count_with_anti_affinity_violation(self, mock_host_passes):
+        """Check failed count after violation of the late affinity check.
+        https://bugs.launchpad.net/nova/+bug/1996732
+        """
+
+        created_group = self.api.post_server_groups(self.anti_affinity)
+        flavor = self.api.get_flavors()[2]
+
+        # Ensure two instances are scheduled on the same host
+        with utils.temporary_mutation(self.admin_api, microversion='2.53'):
+            compute2_service_id = self.admin_api.get_services(
+            host=self.compute2.host, binary='nova-compute')[0]['id']
+            self.admin_api.put_service(compute2_service_id,
+                                        {'status': 'disabled'})
+
+        self._boot_a_server_to_group(created_group, flavor=flavor)
+
+        # Expects GroupAffinityViolation exception
+        failed_server = self._boot_a_server_to_group(created_group,
+                                                     flavor=flavor,
+                                                     expected_status='ERROR')
+
+        self.assertEqual('Exceeded maximum number of retries. Exhausted all '
+                         'hosts available for retrying build failures for '
+                         'instance %s.' % failed_server['id'],
+                         failed_server['fault']['message'])
+
+        ctxt = context.get_admin_context()
+        computes = objects.ComputeNodeList.get_all(ctxt)
+
+        for node in computes:
+            self.assertEqual(node.stats.get('failed_builds'), '0')
+
 
 class ServerGroupAffinityConfTest(ServerGroupTestBase):
     api_major_version = 'v2.1'
@@ -629,10 +696,8 @@ class ServerGroupTestV215(ServerGroupTestV21):
         # Start additional host to test evacuation
         compute3 = self.start_service('compute', host='host3')
 
-        post = {'evacuate': {}}
-        self.admin_api.post_server_action(servers[1]['id'], post)
-        self._wait_for_migration_status(servers[1], ['done'])
-        evacuated_server = self._wait_for_state_change(servers[1], 'ACTIVE')
+        evacuated_server = self._evacuate_server(
+            servers[1], expected_migration_status='done')
 
         # check that the server is evacuated
         self.assertNotEqual(evacuated_server['OS-EXT-SRV-ATTR:host'],
@@ -652,11 +717,9 @@ class ServerGroupTestV215(ServerGroupTestV21):
         # Set forced_down on the host to ensure nova considers the host down.
         self._set_forced_down(host, True)
 
-        post = {'evacuate': {}}
-        self.admin_api.post_server_action(servers[1]['id'], post)
-        self._wait_for_migration_status(servers[1], ['error'])
-        server_after_failed_evac = self._wait_for_state_change(
-            servers[1], 'ERROR')
+        server_after_failed_evac = self._evacuate_server(
+            servers[1], expected_state='ERROR',
+            expected_migration_status='error')
 
         # assert that after a failed evac the server active on the same host
         # as before
@@ -672,11 +735,9 @@ class ServerGroupTestV215(ServerGroupTestV21):
         # Set forced_down on the host to ensure nova considers the host down.
         self._set_forced_down(host, True)
 
-        post = {'evacuate': {}}
-        self.admin_api.post_server_action(servers[1]['id'], post)
-        self._wait_for_migration_status(servers[1], ['error'])
-        server_after_failed_evac = self._wait_for_state_change(
-            servers[1], 'ERROR')
+        server_after_failed_evac = self._evacuate_server(
+            servers[1], expected_state='ERROR',
+            expected_migration_status='error')
 
         # assert that after a failed evac the server active on the same host
         # as before
@@ -814,10 +875,8 @@ class ServerGroupTestV215(ServerGroupTestV21):
         # Set forced_down on the host to ensure nova considers the host down.
         self._set_forced_down(host, True)
 
-        post = {'evacuate': {}}
-        self.admin_api.post_server_action(servers[1]['id'], post)
-        self._wait_for_migration_status(servers[1], ['done'])
-        evacuated_server = self._wait_for_state_change(servers[1], 'ACTIVE')
+        evacuated_server = self._evacuate_server(
+            servers[1], expected_migration_status='done')
 
         # Note(gibi): need to get the server again as the state of the instance
         # goes to ACTIVE first then the host of the instance changes to the
@@ -965,7 +1024,7 @@ class TestAntiAffinityLiveMigration(test.TestCase,
     def setUp(self):
         super(TestAntiAffinityLiveMigration, self).setUp()
         # Setup common fixtures.
-        self.useFixture(policy_fixture.RealPolicyFixture())
+        self.useFixture(nova_fixtures.RealPolicyFixture())
         self.useFixture(nova_fixtures.NeutronFixture(self))
         self.useFixture(nova_fixtures.GlanceFixture(self))
         self.useFixture(func_fixtures.PlacementFixture())
@@ -1033,7 +1092,7 @@ class TestAntiAffinityLiveMigration(test.TestCase,
                                self.admin_api.post_server_action,
                                server['id'], body)
         self.assertEqual(400, ex.response.status_code)
-        self.assertIn('No valid host', six.text_type(ex))
+        self.assertIn('No valid host', str(ex))
 
         # Now start up a 3rd compute service and retry the live migration which
         # should work this time.
