@@ -20,14 +20,15 @@ from lxml import etree
 import mock
 from oslo_utils.fixture import uuidsentinel as uuids
 from oslo_utils import units
-import six
 
 from nova.compute import power_state
 from nova import exception
 from nova.network import model as network_model
 from nova import objects
 from nova import test
-from nova.tests.unit.virt.libvirt import fakelibvirt
+from nova.tests import fixtures as nova_fixtures
+from nova.tests.fixtures import libvirt as fakelibvirt
+from nova.tests.unit.virt.libvirt import test_driver
 from nova.virt.libvirt import config as vconfig
 from nova.virt.libvirt import guest as libvirt_guest
 from nova.virt.libvirt import host
@@ -80,14 +81,51 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
         get_volume_config = mock.MagicMock()
         mock_guest.get_xml_desc.return_value = '<domain></domain>'
 
-        migration.get_updated_guest_xml(mock_guest, data, get_volume_config)
+        instance = objects.Instance(**test_driver._create_test_instance())
+        migration.get_updated_guest_xml(instance, mock_guest, data,
+                                        get_volume_config)
         mock_graphics.assert_called_once_with(mock.ANY, data)
         mock_serial.assert_called_once_with(mock.ANY, data)
-        mock_volume.assert_called_once_with(mock.ANY, data, get_volume_config)
+        mock_volume.assert_called_once_with(
+            mock.ANY, data, instance, get_volume_config)
         mock_perf_events_xml.assert_called_once_with(mock.ANY, data)
         mock_memory_backing.assert_called_once_with(mock.ANY, data)
         self.assertEqual(1, mock_tostring.called)
 
+    def test_update_quota_xml(self):
+        old_xml = """<domain>
+                         <name>fake-instance</name>
+                         <cputune>
+                             <shares>42</shares>
+                             <period>1337</period>
+                         </cputune>
+                     </domain>"""
+        instance = objects.Instance(**test_driver._create_test_instance())
+        new_xml = migration._update_quota_xml(instance,
+                                              etree.fromstring(old_xml))
+        new_xml = etree.tostring(new_xml, encoding='unicode')
+        self.assertXmlEqual(
+            """<domain>
+                   <name>fake-instance</name>
+                   <cputune>
+                       <period>1337</period>
+                   </cputune>
+               </domain>""", new_xml)
+
+    def test_update_quota_xml_empty_cputune(self):
+        old_xml = """<domain>
+                         <name>fake-instance</name>
+                         <cputune>
+                             <shares>42</shares>
+                         </cputune>
+                     </domain>"""
+        instance = objects.Instance(**test_driver._create_test_instance())
+        new_xml = migration._update_quota_xml(instance,
+                                              etree.fromstring(old_xml))
+        new_xml = etree.tostring(new_xml, encoding='unicode')
+        self.assertXmlEqual('<domain><name>fake-instance</name></domain>',
+                            new_xml)
+
     def test_update_device_resources_xml_vpmem(self):
         # original xml for vpmems, /dev/dax0.1 and /dev/dax0.2 here
         # are vpmem device path on source host
@@ -354,8 +392,15 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
 
         get_volume_config = mock.MagicMock(return_value=conf)
         doc = etree.fromstring(xml)
-        res = etree.tostring(migration._update_volume_xml(
-            doc, data, get_volume_config), encoding='unicode')
+        res = etree.tostring(
+            migration._update_volume_xml(
+                doc,
+                data,
+                mock.sentinel.instance,
+                get_volume_config
+            ),
+            encoding='unicode'
+        )
         new_xml = xml.replace('ip-1.2.3.4:3260-iqn.abc.12345.opst-lun-X',
                               'ip-1.2.3.4:3260-iqn.cde.67890.opst-lun-Z')
         self.assertXmlEqual(res, new_xml)
@@ -421,10 +466,16 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
 
         get_volume_config = mock.MagicMock(return_value=conf)
         doc = etree.fromstring(xml)
-        res = etree.tostring(migration._update_volume_xml(
-            doc, data, get_volume_config), encoding='unicode')
-        new_xml = xml.replace('sdb',
-                              'sdc')
+        res = etree.tostring(
+            migration._update_volume_xml(
+                doc,
+                data,
+                mock.sentinel.instance,
+                get_volume_config
+            ),
+            encoding='unicode'
+        )
+        new_xml = xml.replace('sdb', 'sdc')
         self.assertXmlEqual(res, new_xml)
 
     def test_update_volume_xml_add_encryption(self):
@@ -510,8 +561,15 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
 
         get_volume_config = mock.MagicMock(return_value=conf)
         doc = etree.fromstring(xml)
-        res = etree.tostring(migration._update_volume_xml(
-            doc, data, get_volume_config), encoding='unicode')
+        res = etree.tostring(
+            migration._update_volume_xml(
+                doc,
+                data,
+                mock.sentinel.instance,
+                get_volume_config
+            ),
+            encoding='unicode'
+        )
         self.assertXmlEqual(res, new_xml)
 
     def test_update_volume_xml_update_encryption(self):
@@ -578,8 +636,15 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
 
         get_volume_config = mock.MagicMock(return_value=conf)
         doc = etree.fromstring(xml)
-        res = etree.tostring(migration._update_volume_xml(
-            doc, data, get_volume_config), encoding='unicode')
+        res = etree.tostring(
+            migration._update_volume_xml(
+                doc,
+                data,
+                mock.sentinel.instance,
+                get_volume_config
+            ),
+            encoding='unicode'
+        )
         new_xml = xml.replace(uuids.encryption_secret_uuid_old,
                               uuids.encryption_secret_uuid_new)
         self.assertXmlEqual(res, new_xml)
@@ -656,6 +721,7 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
     <source type="file"/>
     <access mode="shared"/>
     <allocation mode="immediate"/>
+    <discard />
   </memoryBacking>
 </domain>"""
         doc = etree.fromstring(xml)
@@ -679,6 +745,7 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
     <source type="file"/>
     <access mode="shared"/>
     <allocation mode="immediate"/>
+    <discard />
   </memoryBacking>
 </domain>""")
 
@@ -691,78 +758,6 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
     <source type="file"/>
     <access mode="shared"/>
     <allocation mode="immediate"/>
-  </memoryBacking>
-</domain>"""
-        doc = etree.fromstring(xml)
-        res = etree.tostring(migration._update_memory_backing_xml(doc, data),
-                             encoding='unicode')
-
-        self.assertXmlEqual(res, """<domain>
-  <memoryBacking>
-    <source type="file"/>
-    <access mode="shared"/>
-    <allocation mode="immediate"/>
-  </memoryBacking>
-</domain>""")
-
-    def test_update_memory_backing_discard_add(self):
-        data = objects.LibvirtLiveMigrateData(
-            dst_wants_file_backed_memory=True, file_backed_memory_discard=True)
-
-        xml = """<domain>
-  <memoryBacking>
-    <source type="file"/>
-    <access mode="shared"/>
-    <allocation mode="immediate"/>
-  </memoryBacking>
-</domain>"""
-        doc = etree.fromstring(xml)
-        res = etree.tostring(migration._update_memory_backing_xml(doc, data),
-                             encoding='unicode')
-
-        self.assertXmlEqual(res, """<domain>
-  <memoryBacking>
-    <source type="file"/>
-    <access mode="shared"/>
-    <allocation mode="immediate"/>
-    <discard />
-  </memoryBacking>
-</domain>""")
-
-    def test_update_memory_backing_discard_remove(self):
-        data = objects.LibvirtLiveMigrateData(
-            dst_wants_file_backed_memory=True,
-            file_backed_memory_discard=False)
-
-        xml = """<domain>
-  <memoryBacking>
-    <source type="file"/>
-    <access mode="shared"/>
-    <allocation mode="immediate"/>
-    <discard />
-  </memoryBacking>
-</domain>"""
-        doc = etree.fromstring(xml)
-        res = etree.tostring(migration._update_memory_backing_xml(doc, data),
-                             encoding='unicode')
-
-        self.assertXmlEqual(res, """<domain>
-  <memoryBacking>
-    <source type="file"/>
-    <access mode="shared"/>
-    <allocation mode="immediate"/>
-  </memoryBacking>
-</domain>""")
-
-    def test_update_memory_backing_discard_keep(self):
-        data = objects.LibvirtLiveMigrateData(
-            dst_wants_file_backed_memory=True, file_backed_memory_discard=True)
-
-        xml = """<domain>
-  <memoryBacking>
-    <source type="file"/>
-    <access mode="shared"/>
-    <allocation mode="immediate"/>
     <discard />
   </memoryBacking>
 </domain>"""
@@ -966,8 +961,7 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
         ex = self.assertRaises(exception.NovaException,
                                migration._update_vif_xml,
                                doc, data, get_vif_config)
-        self.assertIn('Unable to find MAC address in interface XML',
-                      six.text_type(ex))
+        self.assertIn('Unable to find MAC address in interface XML', str(ex))
 
     def test_update_vif_xml_no_matching_vif(self):
         """Tests that the vif in the migrate data is not found in the existing
@@ -991,7 +985,7 @@ class UtilityMigrationTestCase(test.NoDBTestCase):
         doc = etree.fromstring(original_xml)
         ex = self.assertRaises(KeyError, migration._update_vif_xml,
                                doc, data, get_vif_config)
-        self.assertIn("ca:fe:de:ad:be:ef", six.text_type(ex))
+        self.assertIn("ca:fe:de:ad:be:ef", str(ex))
 
     def test_update_vif_xml_lower_case_mac(self):
         """Tests that the vif in the migrate data is not found in the existing
@@ -1039,7 +1033,7 @@ class MigrationMonitorTestCase(test.NoDBTestCase):
     def setUp(self):
         super(MigrationMonitorTestCase, self).setUp()
 
-        self.useFixture(fakelibvirt.FakeLibvirtFixture())
+        self.useFixture(nova_fixtures.LibvirtFixture())
 
         flavor = objects.Flavor(memory_mb=2048,
                                 swap=0,
@@ -1066,12 +1060,12 @@ class MigrationMonitorTestCase(test.NoDBTestCase):
             'image_ref': '155d900f-4e14-4e4c-a73d-069cbf4541e6',
             'root_gb': 10,
             'ephemeral_gb': 20,
-            'instance_type_id': '5',  # m1.small
+            'instance_type_id': flavor.id,
+            'flavor': flavor,
             'extra_specs': {},
             'system_metadata': {
                 'image_disk_format': 'raw',
             },
-            'flavor': flavor,
             'new_flavor': None,
             'old_flavor': None,
             'pci_devices': objects.PciDeviceList(),
