@@ -18,7 +18,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import errno
 import grp
 import os
 import pwd
@@ -185,64 +184,6 @@ def create_ploop_image(
     nova.privsep.libvirt.ploop_init(size, disk_format, fs_type, disk_path)
 
 
-def pick_disk_driver_name(
-    hypervisor_version: int, is_block_dev: bool = False,
-) -> ty.Optional[str]:
-    """Pick the libvirt primary backend driver name
-
-    If the hypervisor supports multiple backend drivers we have to tell libvirt
-    which one should be used.
-
-    Xen supports the following drivers: "tap", "tap2", "phy", "file", or
-    "qemu", being "qemu" the preferred one. Qemu only supports "qemu".
-
-    :param is_block_dev:
-    :returns: driver_name or None
-    """
-    if CONF.libvirt.virt_type == "xen":
-        if is_block_dev:
-            return "phy"
-        else:
-            # 4002000 == 4.2.0
-            if hypervisor_version >= 4002000:
-                try:
-                    nova.privsep.libvirt.xend_probe()
-                except OSError as exc:
-                    if exc.errno == errno.ENOENT:
-                        LOG.debug("xend is not found")
-                        # libvirt will try to use libxl toolstack
-                        return 'qemu'
-                    else:
-                        raise
-                except processutils.ProcessExecutionError:
-                    LOG.debug("xend is not started")
-                    # libvirt will try to use libxl toolstack
-                    return 'qemu'
-            # libvirt will use xend/xm toolstack
-            try:
-                out, err = processutils.execute('tap-ctl', 'check',
-                                                check_exit_code=False)
-                if out == 'ok\n':
-                    # 4000000 == 4.0.0
-                    if hypervisor_version > 4000000:
-                        return "tap2"
-                    else:
-                        return "tap"
-                else:
-                    LOG.info("tap-ctl check: %s", out)
-            except OSError as exc:
-                if exc.errno == errno.ENOENT:
-                    LOG.debug("tap-ctl tool is not installed")
-                else:
-                    raise
-            return "file"
-    elif CONF.libvirt.virt_type in ('kvm', 'qemu'):
-        return "qemu"
-    else:
-        # UML doesn't want a driver_name set
-        return None
-
-
 def get_disk_size(path: str, format: ty.Optional[str] = None) -> int:
     """Get the (virtual) size of a disk image
 
@@ -404,11 +345,7 @@ def find_disk(guest: libvirt_guest.Guest) -> ty.Tuple[str, ty.Optional[str]]:
         raise RuntimeError(_("Can't retrieve root device path "
                              "from instance libvirt configuration"))
 
-    # This is a legacy quirk of libvirt/xen. Everything else should
-    # report the on-disk format in type.
-    if disk_format == 'aio':
-        disk_format = 'raw'
-    return (disk_path, disk_format)
+    return disk_path, disk_format
 
 
 def get_disk_type_from_path(path: str) -> ty.Optional[str]:
@@ -589,6 +526,9 @@ def get_cpu_model_from_arch(arch: str) -> str:
         mode = 'qemu32'
     elif arch == obj_fields.Architecture.PPC64LE:
         mode = 'POWER8'
+    # TODO(chateaulav): Testing of emulated archs ongoing
+    # elif arch == obj_fields.Architecture.MIPSEL:
+    #     mode = '24Kf-mips-cpu'
     # NOTE(kevinz): In aarch64, cpu model 'max' will offer the capabilities
     # that all the stuff it can currently emulate, both for "TCG" and "KVM"
     elif arch == obj_fields.Architecture.AARCH64:
@@ -631,6 +571,7 @@ def get_default_machine_type(arch: str) -> ty.Optional[str]:
     default_mtypes = {
         obj_fields.Architecture.ARMV7: "virt",
         obj_fields.Architecture.AARCH64: "virt",
+        obj_fields.Architecture.PPC64LE: "pseries",
         obj_fields.Architecture.S390: "s390-ccw-virtio",
         obj_fields.Architecture.S390X: "s390-ccw-virtio",
         obj_fields.Architecture.I686: "pc",
@@ -640,22 +581,37 @@ def get_default_machine_type(arch: str) -> ty.Optional[str]:
 
 
 def mdev_name2uuid(mdev_name: str) -> str:
-    """Convert an mdev name (of the form mdev_<uuid_with_underscores>) to a
-    uuid (of the form 8-4-4-4-12).
+    """Convert an mdev name (of the form mdev_<uuid_with_underscores> or
+    mdev_<uuid_with_underscores>_<pciaddress>) to a uuid
+    (of the form 8-4-4-4-12).
+
+    :param mdev_name: the name of the mdev to parse the UUID from
+    :returns: string containing the uuid
     """
-    return str(uuid.UUID(mdev_name[5:].replace('_', '-')))
+    mdev_uuid = mdev_name[5:].replace('_', '-')
+    # Unconditionnally remove the PCI address from the name
+    mdev_uuid = mdev_uuid[:36]
+    return str(uuid.UUID(mdev_uuid))
 
 
-def mdev_uuid2name(mdev_uuid: str) -> str:
-    """Convert an mdev uuid (of the form 8-4-4-4-12) to a name (of the form
-    mdev_<uuid_with_underscores>).
+def mdev_uuid2name(mdev_uuid: str, parent: str = None) -> str:
+    """Convert an mdev uuid (of the form 8-4-4-4-12) and optionally its parent
+    device to a name (of the form mdev_<uuid_with_underscores>[_<pciid>]).
+
+    :param mdev_uuid: the uuid of the mediated device
+    :param parent: the parent device id for the mediated device
+    :returns: name of the mdev to reference in libvirt
     """
-    return "mdev_" + mdev_uuid.replace('-', '_')
+    name = "mdev_" + mdev_uuid.replace('-', '_')
+    if parent and parent.startswith('pci_'):
+        name = name + parent[4:]
+    return name
 
 
 def get_flags_by_flavor_specs(flavor: 'objects.Flavor') -> ty.Set[str]:
     req_spec = objects.RequestSpec(flavor=flavor)
-    resource_request = scheduler_utils.ResourceRequest(req_spec)
+    resource_request = scheduler_utils.ResourceRequest.from_request_spec(
+        req_spec)
     required_traits = resource_request.all_required_traits
 
     flags = [TRAITS_CPU_MAPPING[trait] for trait in required_traits
